#define OPTIMIZE_ROM_CALLS
#define SAVE_SCREEN
#include <tigcclib.h>

#define lcd_width 160
#define lcd_height 100

short _ti89;
short _ti92plus;

//========================GAME GLOBALS====================
char *dark = 0;
char *light = 0;
INT_HANDLER save_int_1;
//========================CLASS BALL======================
int ball_x = lcd_width/2 - 4;
int ball_y = lcd_height/2 - 4;
int ball_Vx = 1;
int ball_Vy = 1;
//========================CLASS PADDLE====================
int paddle_length[] = {25, 25};
int paddle_x[] = {lcd_width/2-8, lcd_width/2-8};
int paddle_y[] = {lcd_height-4, 0};
//========================CLASS GAME======================
int game_speed = 4000;

//=======================PICTURES=========================
static unsigned char ball[2][7] = 
{{0x1C, 0x32, 0x75, 0x79, 0x6F, 0x36, 0x1C},
 {0x1C, 0x3A, 0x79, 0x7F, 0x7F, 0x3E, 0x1C}};

//=======================PROTOTYPES==================
void init(void);
void start(void);
void stop(void);
void paint(void);
void end(void);

//=======================MAIN FUNCTION=====================
void _main(void)
{ 
	int count;
	init();
	start();
	for(;;)
	{
		paddle_x[1] = ball_x;
		//-----------------------MOVE BALL---------------------
		ball_x += ball_Vx;
		ball_y += ball_Vy;
		//--------------------CHECK IF HIT A WALL--------------
		if(ball_x >= lcd_width-7)
		{
			ball_Vx *= -1;
			ball_x = lcd_width-7;
		}
		if(ball_x <= 0) 
		{
			ball_Vx *= -1;
			ball_x = 0;
		}
		//-------------CHECK FOR PADDLE/BALL CONTACT-----------
		if(ball_y+6 == lcd_height-4 && ball_x+6 >= paddle_x[0] && ball_x <= paddle_x[0]+paddle_length[0])
		{
			ball_Vy *= -1;
			if(ball_x+3-paddle_x[0] < paddle_length[0]/2)
				ball_Vx = -4+4*(ball_x+6-paddle_x[0])/(paddle_length[0]/2);
			if(ball_x+3-paddle_x[0] > paddle_length[0]/2)
				ball_Vx = 4*(ball_x-paddle_x[0]-paddle_length[0]/2)/(paddle_length[0]/2);
		}
		if(ball_y == 4 && ball_x+6 >= paddle_x[1] && ball_x <= paddle_x[1]+paddle_length[1])
			ball_Vy *= -1;
		//---------------------IF YOU LOSE BALL----------------
		if(ball_y <= 0) 
		{
			ball_y = lcd_height/2 - 4;
		}
		if(ball_y >= lcd_height-7)
		{
			ball_y = lcd_height/2 - 4;
		}
		//-----------------------GET INPUT---------------------
		if(_rowread(0xFE)&0x2)	//Left
		{
			paddle_x[0] -= 2;
			if(paddle_x[0] < 0) 
				paddle_x[0] = 0; 
		}
		if(_rowread(0xFE)&0x8)	//Right
		{
			paddle_x[0] += 2;
			if(paddle_x[0] > lcd_width-paddle_length[0]-1) 
				paddle_x[0] = lcd_width-paddle_length[0]-1; 
		}
		if(_rowread(0xDF)&0x80)	//F1
		{
			paddle_x[1] -= 2;
			if(paddle_x[1] < 0) 
				paddle_x[1] = 0; 
		}
		if(_rowread(0xF7)&0x80)	//F3
		{
			paddle_x[1] += 2;
			if(paddle_x[1] > lcd_width-paddle_length[1]-1) 
				paddle_x[1] = lcd_width-paddle_length[1]-1;  
		}
		if(_rowread(0xBF)&0x1)	//Esc
			break;
		paint();
		for(count = 0; count < game_speed; count++);
	}
	stop();
	end();
}

void init(void)
{
	dark = malloc(3840);
	light = malloc(3840);
	save_int_1 = GetIntVec (AUTO_INT_1);
	SetIntVec(AUTO_INT_1, DUMMY_HANDLER);
}

void start(void)
{
	GrayMode(GRAY_ON);
}

void stop(void)
{
	GrayMode(GRAY_OFF);
	PortRestore();
}

void paint(void)
{
	int count;
	//------------------Light Plane-----------------
	PortSet(light, 239, 127);
	clrscr();
	//Draw Ball
	Sprite8(ball_x, ball_y, 7, ball[0], light, A_NORMAL);
	//Draw Paddles
	for(count = 0; count < 2; count++)
	{
		DrawLine(paddle_x[count]+1, paddle_y[count], paddle_x[count] + paddle_length[count]-1, paddle_y[count], A_NORMAL);
		DrawLine(paddle_x[count], paddle_y[count]+1, paddle_x[count] + paddle_length[count], paddle_y[count]+1, A_NORMAL);
		DrawLine(paddle_x[count], paddle_y[count]+2, paddle_x[count] + paddle_length[count], paddle_y[count]+2, A_NORMAL);
		DrawLine(paddle_x[count]+1, paddle_y[count]+3, paddle_x[count] + paddle_length[count] - 1, paddle_y[count]+3, A_NORMAL);
	}
	//------------------Dark Plane------------------
	PortSet(dark, 239, 127);
	clrscr();
	//Draw Net
	DrawLine(0, lcd_height/2, lcd_width, lcd_height/2, A_NORMAL);
	//Draw Ball
	Sprite8(ball_x, ball_y, 7, ball[1], dark, A_NORMAL);
	//Draw Paddles
	for(count = 0; count < 2; count++)
	{
		DrawLine(paddle_x[count]+1, paddle_y[count], paddle_x[count] + paddle_length[count]-1, paddle_y[count], A_NORMAL);
		DrawLine(paddle_x[count], paddle_y[count]+1, paddle_x[count], paddle_y[count]+2, A_NORMAL);
		DrawLine(paddle_x[count]+paddle_length[count], paddle_y[count]+1, paddle_x[count]+paddle_length[count], paddle_y[count]+2, A_NORMAL);
		DrawLine(paddle_x[count]+1, paddle_y[count]+3, paddle_x[count] + paddle_length[count] - 1, paddle_y[count]+3, A_NORMAL);
	}
	//Copy to real screen
	memcpy(GetPlane(0),light,LCD_SIZE);
   memcpy(GetPlane(1),dark,LCD_SIZE);
}

void end(void)
{
	SetIntVec(AUTO_INT_1, save_int_1);
	free(light);
	free(dark);
}
